// Vertex Shader wireframe models

#version 330 core

uniform mat4 perspective;
uniform mat4 mv;

uniform int hasBones;

uniform mat4 boneTransform[20];
uniform mat4 boneNormalTransform[20];

layout(location=10) in  ivec2 bone_influence;
layout(location=11) in  vec2 bone_weight;

layout(location=1) in vec3 position;
layout(location=2) in vec3 normal;
layout(location=3) in vec3 tangent;
layout(location=4) in vec3 bitangent;

out vec4 mvNormal;
out vec4 mvTangent;
out vec4 mvBitangent;
out vec4 mvPosition;

void main(void)
{
	if (hasBones==1)
	{
		vec4 p1 = (boneTransform[bone_influence.x] * vec4(position,1.0)) * bone_weight.x;
		vec4 p = p1+(boneTransform[bone_influence.y] * vec4(position,1.0)) * bone_weight.y;
		mvPosition = mv * p;

		mat4 normalMatrix = transpose(inverse(mv));

		vec4 n1 = (boneNormalTransform[bone_influence.x] * vec4(normal,1.0)) * bone_weight.x;
		vec4 n = n1+(boneNormalTransform[bone_influence.y] * vec4(normal,1.0)) * bone_weight.y;
		mvNormal = normalMatrix * n;

		vec4 t1 = (boneNormalTransform[bone_influence.x] * vec4(tangent,1.0)) * bone_weight.x;
		vec4 t = t1+(boneNormalTransform[bone_influence.y] * vec4(tangent,1.0)) * bone_weight.y;
		mvTangent = normalMatrix * t;

		vec4 b1 = (boneNormalTransform[bone_influence.x] * vec4(bitangent,1.0)) * bone_weight.x;
		vec4 b = b1+(boneNormalTransform[bone_influence.y] * vec4(bitangent,1.0)) * bone_weight.y;
		mvBitangent = normalMatrix * b;

		gl_Position = perspective * mvPosition;
	}
	else
	{
		mvPosition = mv * vec4(position,1.0);
		mat4 normalMatrix = transpose(inverse(mv));
		mvNormal = normalMatrix * vec4(normal,1.0);
		mvTangent = normalMatrix * vec4(tangent,1.0);
		mvBitangent = normalMatrix * vec4(bitangent,1.0);
		gl_Position = perspective * mvPosition;
	}
}